function [] = main_ol(u_max, Q, R, T, x_init, N)
clearvars -except u_max Q R T x_init N
% close all
clc

import casadi.*

% N = 50; % number of control intervals
nx = 2;
nu = 1;

% Declare model variables
x = MX.sym('x',nx,1);
u = MX.sym('u',nu,1);

x0_bar = x_init;

% Model equations
xdot = dynamics(x, u);

% Formulate discrete time dynamics
% Fixed step Runge-Kutta 4 integrator
M = 1; % RK4 steps per interval
DT = T/N/M;
f = Function('f', {x, u}, {xdot});
X0 = MX.sym('X0', nx,1);
U = MX.sym('U',nu,1);
X = X0;

for j=1:M
    k1 = f(X, U);
    k2 = f(X + DT/2 * k1, U);
    k3 = f(X + DT/2 * k2, U);
    k4 = f(X + DT * k3, U);
    X=X+DT/6*(k1 +2*k2 +2*k3 +k4);
end

F = Function('F', {X0, U}, {X});

% Evaluate at a test point
Fk = F([0.2; 0.3],0.4);
disp(Fk)

% Start with an empty NLP
w={};
w0 = [];
lbw = [];
ubw = [];
J = 0;
g={};
lbg = [];
ubg = [];

X0 = MX.sym('X0', nx, 1);
w0 = [w0;  0.1; 0.1];
w = {w{:}, X0};
lbw = [lbw; -inf; -inf];
ubw = [ubw;  inf;  inf];

g = {g{:}, X0 };
lbg = [lbg; x0_bar];
ubg = [ubg; x0_bar];

% Formulate the NLP
Xk = x0_bar;
for k=0:N-1
    % New NLP variable for the control
    Uk = MX.sym(['U_' num2str(k)], nu, 1);
    w = {w{:}, Uk};
    w0 = [w0;  0.1];
    lbw = [lbw; -u_max];
    ubw = [ubw;  u_max];

    % Integrate till the end of the interval
    Xk_end =  F(Xk, Uk);
    J = J+ 1/2*(Xk.'*Q*Xk + R*Uk^2);

    % New NLP variable for state at end of interval
    Xk = MX.sym(['X_' num2str(k+1)], nx, 1);
    w = {w{:}, Xk};
    lbw = [lbw; -inf; -inf];
    ubw = [ubw;  inf;  inf];
    w0 = [w0; 0.1; 0.1];

    % Add equality constraint
    g = {g{:}, Xk_end-Xk};
    lbg = [lbg; 0; 0];
    ubg = [ubg; 0; 0];
end

J = J+ 1/2*Xk.'*Q*Xk;

g = vertcat(g{:});
w = vertcat(w{:});

% Create an NLP solver
prob = struct('f', J, 'x', w, 'g', g);
solver = nlpsol('solver', 'ipopt', prob);

% Solve the NLP
sol = solver('x0', w0,...
            'lbg', lbg, 'ubg', ubg, 'lbx', lbw, 'ubx', ubw);
        
w_opt = full(sol.x);

% figure()
% Plot the solution
x1_opt = [w_opt(1:3:end)];
x2_opt = [w_opt(2:3:end)];
u_opt = w_opt(3:3:end);
tgrid = linspace(0, T, N+1);
clf;
subplot(211)
hold all
plot(tgrid, x1_opt, 'b-')
plot(tgrid, x2_opt, 'b-')
grid on
xlabel('t')
ylabel('x')
subplot(212)
hold all
stairs(tgrid, [u_opt; nan],'b')
grid on
xlabel('t')
ylabel('u')